/*
 * Copyright (c) Jupyter Development Team.
 * Distributed under the terms of the Modified BSD License.
 */
import { ChatWidget } from '@jupyter/chat';
import { ABCWidgetFactory } from '@jupyterlab/docregistry';
import { Signal } from '@lumino/signaling';
import { CollaborativeChatModel } from './model';
import { CollaborativeChatWidget } from './widget';
/**
 * The object provided by the chatDocument extension.
 * It is used to set the current config (from settings) to newly created chat widget,
 * and to propagate every changes to the existing chat widgets.
 */
export class WidgetConfig {
    /**
     * The constructor of the WidgetConfig.
     */
    constructor(config) {
        this.configChanged = new Signal(this);
        this.config = config;
        this.configChanged.connect((_, config) => {
            this.config = { ...this.config, ...config };
        });
    }
}
/**
 * A widget factory to create new instances of CollaborativeChatWidget.
 */
export class ChatWidgetFactory extends ABCWidgetFactory {
    /**
     * Constructor of ChatWidgetFactory.
     *
     * @param options Constructor options
     */
    constructor(options) {
        super(options);
        this._themeManager = options.themeManager;
        this._rmRegistry = options.rmRegistry;
    }
    /**
     * Create a new widget given a context.
     *
     * @param context Contains the information of the file
     * @returns The widget
     */
    createNewWidget(context) {
        context.chatModel = context.model;
        context.rmRegistry = this._rmRegistry;
        context.themeManager = this._themeManager;
        return new CollaborativeChatWidget({
            context,
            content: new ChatWidget(context)
        });
    }
}
export class CollaborativeChatModelFactory {
    constructor(options) {
        this.collaborative = true;
        this._disposed = false;
        this._user = options.user;
        this._widgetConfig = options.widgetConfig;
    }
    /**
     * The name of the model.
     *
     * @returns The name
     */
    get name() {
        return 'chat';
    }
    /**
     * The content type of the file.
     *
     * @returns The content type
     */
    get contentType() {
        return 'chat';
    }
    /**
     * The format of the file.
     *
     * @returns the file format
     */
    get fileFormat() {
        return 'text';
    }
    /**
     * Get whether the model factory has been disposed.
     *
     * @returns disposed status
     */
    get isDisposed() {
        return this._disposed;
    }
    /**
     * Dispose the model factory.
     */
    dispose() {
        this._disposed = true;
    }
    /**
     * Get the preferred language given the path on the file.
     *
     * @param path path of the file represented by this document model
     * @returns The preferred language
     */
    preferredLanguage(path) {
        return '';
    }
    /**
     * Create a new instance of CollaborativeChatModel.
     *
     * @param languagePreference Language
     * @param modelDB Model database
     * @returns The model
     */
    createNew(options) {
        return new CollaborativeChatModel({
            ...options,
            user: this._user,
            widgetConfig: this._widgetConfig
        });
    }
}
