/*
 * Copyright (c) Jupyter Development Team.
 * Distributed under the terms of the Modified BSD License.
 */
import { ChatModel } from '@jupyter/chat';
import { UUID } from '@lumino/coreutils';
import { Signal } from '@lumino/signaling';
import { YChat } from './ychat';
/**
 * The collaborative chat model.
 */
export class CollaborativeChatModel extends ChatModel {
    constructor(options) {
        super(options);
        this.collaborative = true;
        this._onchange = (_, change) => {
            if (change.messageChanges) {
                const msgDelta = change.messageChanges;
                let index = 0;
                msgDelta.forEach(delta => {
                    if (delta.retain) {
                        index += delta.retain;
                    }
                    else if (delta.insert) {
                        const messages = delta.insert.map(ymessage => {
                            const msg = { ...ymessage };
                            msg.sender =
                                this.sharedModel.getUser(ymessage.sender) || ymessage.sender;
                            return msg;
                        });
                        this.messagesInserted(index, messages);
                        index += messages.length;
                    }
                    else if (delta.delete) {
                        this.messagesDeleted(index, delta.delete);
                    }
                });
            }
        };
        this.defaultKernelName = '';
        this.defaultKernelLanguage = '';
        this._dirty = false;
        this._readOnly = false;
        this._disposed = new Signal(this);
        this._contentChanged = new Signal(this);
        this._stateChanged = new Signal(this);
        this._user = options.user || { username: 'user undefined' };
        const { widgetConfig, sharedModel } = options;
        if (sharedModel) {
            this._sharedModel = sharedModel;
        }
        else {
            this._sharedModel = YChat.create();
        }
        this.sharedModel.changed.connect(this._onchange, this);
        this.config = widgetConfig.config;
        widgetConfig.configChanged.connect((_, config) => {
            this.config = config;
        });
    }
    get user() {
        return this._user;
    }
    get sharedModel() {
        return this._sharedModel;
    }
    get contentChanged() {
        return this._contentChanged;
    }
    get stateChanged() {
        return this._stateChanged;
    }
    get dirty() {
        return this._dirty;
    }
    set dirty(value) {
        this._dirty = value;
    }
    get readOnly() {
        return this._readOnly;
    }
    set readOnly(value) {
        this._readOnly = value;
    }
    get disposed() {
        return this._disposed;
    }
    dispose() {
        if (this.isDisposed) {
            return;
        }
        super.dispose();
        this._sharedModel.dispose();
        this._disposed.emit();
        Signal.clearData(this);
    }
    toString() {
        return JSON.stringify({}, null, 2);
    }
    fromString(data) {
        /** */
    }
    toJSON() {
        return JSON.parse(this.toString());
    }
    fromJSON(data) {
        // nothing to do
    }
    addMessage(message) {
        const msg = {
            type: 'msg',
            id: UUID.uuid4(),
            body: message.body,
            time: Date.now() / 1000,
            sender: this._user.username,
            raw_time: true
        };
        // Add the user if it does not exist or has changed
        if (!(this.sharedModel.getUser(this._user.username) === this._user)) {
            this.sharedModel.setUser(this._user);
        }
        this.sharedModel.addMessage(msg);
    }
    updateMessage(id, updatedMessage) {
        const index = this.sharedModel.getMessageIndex(id);
        let message = this.sharedModel.getMessage(index);
        if (message) {
            message.body = updatedMessage.body;
            message.edited = true;
        }
        else {
            let sender;
            if (typeof updatedMessage.sender === 'string') {
                sender = updatedMessage.sender;
            }
            else {
                sender = updatedMessage.sender.username;
            }
            message = {
                type: 'msg',
                id: id || UUID.uuid4(),
                body: updatedMessage.body,
                time: updatedMessage.time || Date.now() / 1000,
                sender: sender,
                edited: true
            };
        }
        this.sharedModel.updateMessage(index, message);
    }
    deleteMessage(id) {
        const index = this.sharedModel.getMessageIndex(id);
        const message = this.sharedModel.getMessage(index);
        if (!message) {
            console.error('The message to delete does not exist');
            return;
        }
        message.body = '';
        message.deleted = true;
        this.sharedModel.updateMessage(index, message);
    }
}
