/*
 * Copyright (c) Jupyter Development Team.
 * Distributed under the terms of the Modified BSD License.
 */
import { ChatWidget } from '@jupyter/chat';
import { PathExt } from '@jupyterlab/coreutils';
import { DocumentWidget } from '@jupyterlab/docregistry';
import { addIcon, closeIcon, CommandToolbarButton, HTMLSelect, launchIcon, PanelWithToolbar, ReactWidget, SidePanel, ToolbarButton } from '@jupyterlab/ui-components';
import { Signal } from '@lumino/signaling';
import React, { useState } from 'react';
import { CommandIDs, chatFileType } from './token';
const MAIN_PANEL_CLASS = 'jp-collab-chat_main-panel';
const SIDEPANEL_CLASS = 'jp-collab-chat-sidepanel';
const ADD_BUTTON_CLASS = 'jp-collab-chat-add';
const OPEN_SELECT_CLASS = 'jp-collab-chat-open';
const SECTION_CLASS = 'jp-collab-chat-section';
const TOOLBAR_CLASS = 'jp-collab-chat-toolbar';
/**
 * DocumentWidget: widget that represents the view or editor for a file type.
 */
export class CollaborativeChatWidget extends DocumentWidget {
    constructor(options) {
        super(options);
        this.addClass(MAIN_PANEL_CLASS);
    }
    /**
     * Dispose of the resources held by the widget.
     */
    dispose() {
        this.context.dispose();
        this.content.dispose();
        super.dispose();
    }
    /**
     * The model for the widget.
     */
    get model() {
        return this.content.model;
    }
}
/**
 * Sidepanel widget including the chats and the add chat button.
 */
export class ChatPanel extends SidePanel {
    /**
     * The constructor of the chat panel.
     */
    constructor(options) {
        super(options);
        this.updateChatNames = async () => {
            const extension = chatFileType.extensions[0];
            this._drive
                .get('.')
                .then(model => {
                const chatsName = model.content
                    .filter(f => f.type === 'file' && f.name.endsWith(extension))
                    .map(f => PathExt.basename(f.name, extension));
                this._chatNamesChanged.emit(chatsName);
            })
                .catch(e => console.error('Error getting the chat files from drive', e));
        };
        /**
         * Handle `change` events for the HTMLSelect component.
         */
        this._chatSelected = (event) => {
            const value = event.target.value;
            if (value === '-') {
                return;
            }
            const index = this.widgets.findIndex(w => w.name === value);
            if (index === -1) {
                this._commands.execute(CommandIDs.openChat, {
                    filepath: `${value}${chatFileType.extensions[0]}`,
                    inSidePanel: true
                });
            }
            else if (!this.widgets[index].isVisible) {
                this.content.expand(index);
            }
            event.target.selectedIndex = 0;
        };
        this._chatNamesChanged = new Signal(this);
        this._config = {};
        this.addClass(SIDEPANEL_CLASS);
        this._commands = options.commands;
        this._drive = options.drive;
        this._rmRegistry = options.rmRegistry;
        this._themeManager = options.themeManager;
        const addChat = new CommandToolbarButton({
            commands: this._commands,
            id: CommandIDs.createChat,
            args: { inSidePanel: true },
            icon: addIcon
        });
        addChat.addClass(ADD_BUTTON_CLASS);
        this.toolbar.addItem('createChat', addChat);
        this._openChat = ReactWidget.create(React.createElement(ChatSelect, { chatNamesChanged: this._chatNamesChanged, handleChange: this._chatSelected.bind(this) }));
        this._openChat.addClass(OPEN_SELECT_CLASS);
        this.toolbar.addItem('openChat', this._openChat);
        const content = this.content;
        content.expansionToggled.connect(this._onExpansionToggled, this);
    }
    /**
     * Getter and setter of the config, propagated to all the chat widgets.
     */
    get config() {
        return this._config;
    }
    set config(value) {
        this._config = { ...this._config, ...value };
        this.widgets.forEach(w => {
            w.model.config = value;
        });
    }
    /**
     * Add a new widget to the chat panel.
     *
     * @param model - the model of the chat widget
     * @param name - the name of the chat.
     */
    addChat(model, name) {
        // Collapse all chats
        const content = this.content;
        for (let i = 0; i < this.widgets.length; i++) {
            content.collapse(i);
        }
        // Create a new widget.
        const widget = new ChatWidget({
            model: model,
            rmRegistry: this._rmRegistry,
            themeManager: this._themeManager
        });
        this.addWidget(new ChatSection({ name, widget, commands: this._commands }));
    }
    /**
     * A message handler invoked on an `'after-show'` message.
     */
    onAfterShow(msg) {
        var _a;
        // Wait for the component to be rendered.
        (_a = this._openChat.renderPromise) === null || _a === void 0 ? void 0 : _a.then(() => this.updateChatNames());
    }
    /**
     * Triggered when a section is toogled. If the section is opened, all others
     * sections are closed.
     */
    _onExpansionToggled(panel, index) {
        if (!this.widgets[index].isVisible) {
            return;
        }
        for (let i = 0; i < this.widgets.length; i++) {
            if (i !== index) {
                panel.collapse(i);
            }
        }
    }
}
/**
 * The chat section containing a chat widget.
 */
class ChatSection extends PanelWithToolbar {
    /**
     * Constructor of the chat section.
     */
    constructor(options) {
        super(options);
        this.addClass(SECTION_CLASS);
        this._name = options.name;
        this.title.label = this._name;
        this.title.caption = this._name;
        this.toolbar.addClass(TOOLBAR_CLASS);
        const moveToMain = new ToolbarButton({
            icon: launchIcon,
            iconLabel: 'Move the chat to the main area',
            className: 'jp-mod-styled',
            onClick: () => {
                this.model.dispose();
                options.commands.execute(CommandIDs.openChat, {
                    filepath: `${this._name}${chatFileType.extensions[0]}`
                });
                this.dispose();
            }
        });
        const closeButton = new ToolbarButton({
            icon: closeIcon,
            iconLabel: 'Close the chat',
            className: 'jp-mod-styled',
            onClick: () => {
                this.model.dispose();
                this.dispose();
            }
        });
        this.toolbar.addItem('collaborativeChat-main', moveToMain);
        this.toolbar.addItem('collaborativeChat-close', closeButton);
        this.addWidget(options.widget);
        options.widget.node.style.height = '100%';
    }
    /**
     * The name of the chat.
     */
    get name() {
        return this._name;
    }
    /**
     * The model of the widget.
     */
    get model() {
        return this.widgets[0].model;
    }
}
/**
 * A component to select a chat from the drive.
 */
function ChatSelect({ chatNamesChanged, handleChange }) {
    const [chatNames, setChatNames] = useState([]);
    // Update the chat list.
    chatNamesChanged.connect((_, chatNames) => {
        setChatNames(chatNames);
    });
    return (React.createElement(HTMLSelect, { onChange: handleChange },
        React.createElement("option", { value: "-" }, "Open a chat"),
        chatNames.map(name => (React.createElement("option", { value: name }, name)))));
}
