/*
 * Copyright (c) Jupyter Development Team.
 * Distributed under the terms of the Modified BSD License.
 */
import { YDocument } from '@jupyter/ydoc';
import { JSONExt } from '@lumino/coreutils';
/**
 * The collaborative chat shared document.
 */
export class YChat extends YDocument {
    /**
     * Create a new collaborative chat model.
     */
    constructor(options) {
        super(options);
        /**
         * Document version
         */
        this.version = '1.0.0';
        this._usersObserver = (event) => {
            const userChange = new Array();
            event.keysChanged.forEach(key => {
                const change = event.changes.keys.get(key);
                if (change) {
                    switch (change.action) {
                        case 'add':
                            userChange.push({
                                key,
                                newValue: this._users.get(key),
                                type: 'add'
                            });
                            break;
                        case 'delete':
                            userChange.push({
                                key,
                                oldValue: change.oldValue,
                                type: 'remove'
                            });
                            break;
                        case 'update':
                            userChange.push({
                                key: key,
                                oldValue: change.oldValue,
                                newValue: this._users.get(key),
                                type: 'change'
                            });
                            break;
                    }
                }
            });
            this._changed.emit({ userChange: userChange });
        };
        this._messagesObserver = (event) => {
            const messageChanges = event.delta;
            this._changed.emit({ messageChanges: messageChanges });
        };
        this._users = this.ydoc.getMap('users');
        this._users.observe(this._usersObserver);
        this._messages = this.ydoc.getArray('messages');
        this._messages.observe(this._messagesObserver);
    }
    /**
     * Static method to create instances on the sharedModel
     *
     * @returns The sharedModel instance
     */
    static create(options) {
        return new YChat(options);
    }
    get users() {
        return JSONExt.deepCopy(this._users.toJSON());
    }
    get messages() {
        return JSONExt.deepCopy(this._messages.toJSON());
    }
    getUser(username) {
        if (!username) {
            return undefined;
        }
        return this._users.get(username);
    }
    setUser(value) {
        this.transact(() => {
            this._users.set(value.username, value);
        });
    }
    getMessage(index) {
        return this._messages.get(index);
    }
    addMessage(value) {
        this.transact(() => {
            this._messages.push([value]);
        });
    }
    updateMessage(index, value) {
        this.transact(() => {
            this._messages.delete(index);
            this._messages.insert(index, [value]);
        });
    }
    getMessageIndex(id) {
        return this._messages.toArray().findIndex(msg => msg.id === id);
    }
    deleteMessage(index) {
        this.transact(() => {
            this._messages.delete(index);
        });
    }
}
